#ifndef _NRF24L01_H_
#define _NRF24L01_H_

#define Pin volatile unsigned long
#define BITBAND(addr, bitnum) ((addr & 0xF0000000)+0x2000000+((addr &0xFFFFF)<<5)+(bitnum<<2))
#define GET_BITBAND_WRITE_POINTER(GPIO,PIN)  (Pin *)BITBAND(((unsigned long)(&(GPIO->ODR))), (PIN))
#define GET_BITBAND_READ_POINTER(GPIO,PIN)  (Pin *)BITBAND(((unsigned long)(&(GPIO->IDR))), (PIN))

#define NRF_R_REGISTER      0X00
#define NRF_W_REGISTER      0X20
#define NRF_R_RX_PAYLOAD    0X61
#define NRF_W_RX_PAYLOAD    0XA0
#define NRF_FLUSH_TX        0XE1
#define NRF_FLUSH_RX        0XE2
#define NRF_REUSE_TX_PL     0XE3
#define NRF_NOP             0XFF

#define NRF_CONFIG          0X00
#define NRF_EN_AA           0X01
#define NRF_EN_RXADDR       0X02
#define NRF_SETUP_AW        0X03
#define NRF_SETUP_RETR      0X04
#define NRF_RF_CH           0X05
#define NRF_RF_SETUP        0X06
#define NRF_STATUS          0X07
#define NRF_OBSERVE_TX      0X08
#define NRF_CD              0X09
#define NRF_RX_ADDR_P0      0X0A
#define NRF_RX_ADDR_P1      0X0B
#define NRF_RX_ADDR_P2      0X0C
#define NRF_RX_ADDR_P3      0X0D
#define NRF_RX_ADDR_P4      0X0E
#define NRF_RX_ADDR_P5      0X0F
#define NRF_TX_ADDR         0X10
#define NRF_RX_PW_P0        0X11
#define NRF_RX_PW_P1        0X12
#define NRF_RX_PW_P2        0X13
#define NRF_RX_PW_P3        0X14
#define NRF_RX_PW_P4        0X15
#define NRF_RX_PW_P5        0X16
#define NRF_FIFO_STATUS     0X17

#define NRF_DATA_LENGTH     32

typedef enum
{
    RX_IT = 6,
    TX_IT = 5,
    MAX_RT_IT = 4,
}NRF_IT_Flag;

typedef enum
{
    TX_MODE = 0,
    RX_MODE = 1,
}NRF_Mode;

typedef struct
{
    GPIO_TypeDef *GPIO_Port;
    unsigned short int GPIO_Pin;
    unsigned char GPIO_AF;
}_gpio_;

typedef struct
{
    _gpio_ CE_Pin;
    _gpio_ CS_Pin;
    _gpio_ MISO_Pin;
    _gpio_ MOSI_Pin;
    _gpio_ CLK_Pin;
    _gpio_ IRQ_Pin;

    SPI_TypeDef *SPI;
    TIM_TypeDef *TIM;

    Pin *CS;
    Pin *CE;
	Pin *IRQ;

    NRF_Mode Mode;
    
    void(*Init_RCC)();
    void(*Init_SPI)();
    void(*Init_TIM)();
    void(*Init_IRQ)();

    void(*TIM_IRQ)();
    void(*NRF_IRQ)();
}_dev_nrf_;

typedef struct
{
    u8 Type : 7;
    u8 MoreData : 1;
    u8 Length;
    u16 TotalLength;
    u8 Content[NRF_DATA_LENGTH-4];
}_nrf_protocol_;

typedef union
{
    u8 Raw[NRF_DATA_LENGTH];
    _nrf_protocol_ Protocol;
}nrf_protocol;

typedef struct
{
    u8 Update_Flag;
    nrf_protocol Data;
}_nrf_data_packet_;

typedef struct
{
    _nrf_data_packet_ TX;
    _nrf_data_packet_ RX[6];
}_nrf_data_;

extern volatile _dev_nrf_ NRF_Device;
extern volatile _nrf_data_ NRF_Data;

extern void Init_NRF(void);
extern u8 NRF_Write(u8 reg, u8 *dat, u8 len);
extern u8 NRF_Read(u8 reg, u8 *dat, u8 len);
extern u8 NRF_Get_Status(void);
extern void NRF_Clear_IT(NRF_IT_Flag IT_Flag);
extern u8 NRF_Check(void);
extern void NRF_Tx_Data(_nrf_data_packet_* packet);
extern void NRF_Rx_Data(_nrf_data_packet_* packet);

#endif
